{ TD2NumberBox ================================================================}

constructor TD2NumberBox.Create(AOwner: TComponent);
begin
  inherited;
  FDecimalDigits := 2;
  FFilterChar := '0123456789.,-+';
  Max := 10;
  VertIncrement := 5;
  HorzIncrement := 1;
  TextW := '0';
  Value := 0;
  AutoCapture := true;
end;

destructor TD2NumberBox.Destroy;
begin
  inherited;
end;

function TD2NumberBox.GetData: Variant;
begin
  Result := Value;
end;

procedure TD2NumberBox.SetData(const Value: Variant);
begin
  if VarIsEvent(Value) then
    OnChange := VariantToEvent(Value)
  else
    Self.Value := Value;
end;

procedure TD2NumberBox.KeyDown(var Key: Word; var KeyChar: System.WideChar;
  Shift: TShiftState);
begin
  case Key of
    VK_UP:   FValue := FValue + HorzIncrement;
    VK_DOWN: FValue := FValue - HorzIncrement;
  else
    inherited;
    Exit;
  end;
  Key := 0;
  Reformat(True);
end;

procedure TD2NumberBox.MouseDown(Button: TMouseButton; Shift: TShiftState;
  X, Y: single);
begin
  inherited;
  if Button = mbLeft then
  begin
    FPressed := true;
    FPressedPos := d2Point(X, Y);
    FPressedVert := false;
    FPressedInc := 0;
  end;
end;

procedure TD2NumberBox.MouseMove(Shift: TShiftState; X, Y, Dx, Dy: single);
begin
  inherited;
  if FPressed then
  begin
    if Abs(X - FPressedPos.X) >= Abs(Y - FPressedPos.Y) then
    begin
      { horz }
      if X > FPressedPos.X then
        FValue := FValue + HorzIncrement
      else
        FValue := FValue - HorzIncrement;
      FPressedInc := X - FPressedPos.X;
      FPressedVert := false;
    end
    else
    begin
      { vert }
      if Y < FPressedPos.Y then
        FValue := FValue + VertIncrement
      else
        FValue := FValue - VertIncrement;
      FPressedInc := X - FPressedPos.X;
      FPressedVert := true;
    end;
    FPressedPos := d2Point(X, Y);
    Reformat(True);
  end;
end;

procedure TD2NumberBox.MouseUp(Button: TMouseButton; Shift: TShiftState; X,
  Y: single);
begin
  inherited;
  if FPressed then
  begin
    FPressed := false;
    Repaint;
    Change;
  end;
end;

procedure TD2NumberBox.Paint;
begin
  inherited ;
end;

procedure TD2NumberBox.PaintChildren;
var
  R: TD2Rect;
begin
  if FPressed then
    FDisableCaret := true;
  inherited;
  if FPressed then
  begin
    Canvas.SetMatrix(AbsoluteMatrix);
    Canvas.Fill.Style := d2BrushSolid;
    Canvas.Fill.SolidColor := $AA505050;
    R := LocalRect;
    if FPressedVert then
    begin
      d2InflateRect(R, -1, -1);
      R.Left := R.Right - 5;
      Canvas.FillRect(R, 1, 1, AllCorners, AbsoluteOpacity);
      d2InflateRect(R, -1, -1);
{      if FPressedInc > 0 then
      begin
        Canvas.Fill.SolidColor := $AA202020;
        R.Top := (d2RectHeight(R) / 2);
        R.Bottom := R.Top + (Height / 2.1);
        Canvas.FillRect(R, 1, 1, AbsoluteOpacity);
      end;
      if FPressedInc < 0 then
      begin
        Canvas.Fill.SolidColor := $AA202020;
        R.Bottom := (d2RectHeight(R) / 2);
        R.Top := R.Bottom - (Height / 2.1);
        Canvas.FillRect(R, 1, 1, AbsoluteOpacity);
      end;}
    end
    else
    begin
      d2InflateRect(R, -1, -1);
      R.Top := R.Bottom - 5;
      Canvas.FillRect(R, 1, 1, AllCorners, AbsoluteOpacity);
      d2InflateRect(R, -1, -1);
{      if FPressedInc > 0 then
      begin
        Canvas.Fill.SolidColor := $AA202020;
        R.Left := (d2RectWidth(R) / 2);
        R.Right := R.Left + (Width / 2.1);
        Canvas.FillRect(R, 1, 1, AbsoluteOpacity);
      end;
      if FPressedInc < 0 then
      begin
        Canvas.Fill.SolidColor := $AA202020;
        R.Right := (d2RectWidth(R) / 2);
        R.Left := R.Right - (Width / 2.1);
        Canvas.FillRect(R, 1, 1, AbsoluteOpacity);
      end;}
    end;
  end;
  if FPressed then
    FDisableCaret := false;
end;

procedure TD2NumberBox.SetMax(const Value: single);
begin
  if FMax <> Value then begin
     FMax := Value;
     Reformat(False);
  end{if};
end;

procedure TD2NumberBox.SetMin(const Value: single);
begin
  if FMin <> Value then begin
     FMin := Value;
     Reformat(False);
  end{if};
end;

procedure TD2NumberBox.SetDecimalDigits(const Value: integer);
begin
  if FDecimalDigits <> Value then begin
     FDecimalDigits := Value;
     Reformat(False);
  end{if};
end;

procedure TD2NumberBox.SetValue(const AValue: single);
begin
  if FValue <> AValue then
  begin
    FValue := AValue;
    Reformat(True);
  end;
end;

procedure TD2NumberBox.SetValueType(const Value: TD2ValueType);
begin
  if FValueType <> Value then begin
     FValueType := Value;
     if (FValueType = d2ValueInteger) then begin
        HorzIncrement := Trunc(HorzIncrement);
        VertIncrement := Trunc(VertIncrement);
     end;
     Reformat(False);
  end{if};
end;

procedure TD2NumberBox.Reformat(const ValueChanged: Boolean);
var
   OldText:  WideString;
begin
  FNeedChange := ValueChanged;
  OldText := FText;
  if FValue > FMax then begin FNeedChange := True; FValue := FMax; end{if};
  if FValue < FMin then begin FNeedChange := True; FValue := FMin; end{if};
  if FValueType = d2ValueInteger then
    FText := IntToStr(Trunc(FValue))
  else
    FText := Format('%.' + IntToStr(FDecimalDigits) +
                     'f', [FValue]                    );
  if FText <> OldText then begin
     SelLength := 0;
     Repaint;
     Change;
  end{if};
end;

procedure TD2NumberBox.SetText(const Value: WideString); // 7777
begin
  if FText <> Value then
  Try
     FValue := StrToFloat(Value);
  Except
  end{try};
  inherited;
end;

//============================= TD2SpinBox ===========================================

constructor TD2SpinBox.Create(AOwner: TComponent);
begin
  inherited;
  FDecimalDigits := 2;
  FFilterChar := '0123456789.,-+';
  TextAlign := d2TextAlignCenter;
  Increment := 1;
  Max := 10;
  TextW := '0';
  Value := 0;
end;

destructor TD2SpinBox.Destroy;
begin
  inherited;
end;

procedure TD2SpinBox.ApplyStyle;
var
  B: TD2Object;
begin
  inherited;
  B := FindResource('minusbutton');
  if (B <> nil) and (B is TD2CustomButton) then
  begin
    FMinus := TD2CustomButton(B);
    FMinus.OnClick := DoMinusClick;
  end;
  B := FindResource('plusbutton');
  if (B <> nil) and (B is TD2CustomButton) then
  begin
    FPlus := TD2CustomButton(B);
    FPlus.OnClick := DoPlusClick;
  end;
end;

procedure TD2SpinBox.FreeStyle;
begin
  inherited;
  FMinus := nil;
  FPlus := nil;
end;

procedure TD2SpinBox.DoMinusClick(Sender: TObject);
begin
   SetFocus;
   FValue := FValue - Increment;
   Reformat(True);
end;

procedure TD2SpinBox.DoPlusClick(Sender: TObject);
begin
   SetFocus;
   FValue := FValue + Increment;
   Reformat(True);
end;

function TD2SpinBox.GetData: Variant;
begin
  Result := Value;
end;

procedure TD2SpinBox.SetData(const Value: Variant);
begin
  if VarIsEvent(Value) then
    OnChange := VariantToEvent(Value)
  else
    Self.Value := Value;
end;

procedure TD2SpinBox.KeyDown(var Key: Word; var KeyChar: System.WideChar; Shift: TShiftState);
begin
  case Key of
    VK_UP: Value := Value + Increment;
    VK_DOWN: Value := Value - Increment;
  else
    inherited;
    Exit;
  end;
  Key := 0;
end;

procedure TD2SpinBox.SetMax(const Value: single);
begin
  if FMax <> Value then begin
     FMax := Value;
     Reformat(False);
  end{if};
end;

procedure TD2SpinBox.SetMin(const Value: single);
begin
  if FMin <> Value then begin
     FMin := Value;
     Reformat(False);
  end{if};
end;

procedure TD2SpinBox.SetValue(const AValue: single);
begin
  if FValue <> AValue then
  begin
    FValue := AValue;
    Reformat(True);
  end;
end;

procedure TD2SpinBox.SetValueType(const Value: TD2ValueType);
begin
  if FValueType <> Value then
  begin
    FValueType := Value;
    if (FValueType = d2ValueInteger) and (Frac(Increment) <> 0) then
      Increment := 1;
    Reformat(False);
  end;
end;

procedure TD2SpinBox.SetDecimalDigits(const Value: integer);
begin
  if FDecimalDigits <> Value then begin
     FDecimalDigits := Value;
     Reformat(False);
  end{if};
end;

procedure TD2SpinBox.Reformat(const ValueChanged: Boolean);
var
   OldText:  WideString;
begin
   FNeedChange := ValueChanged;
   OldText := FText;
   if FValue > FMax then begin FValue := FMax; FNeedChange := True; end{if};
   if FValue < FMin then begin FValue := FMin; FNeedChange := True; end{if};
   if FValueType = d2ValueInteger then
      FText := IntToStr(Trunc(FValue))
   else
      FText := Format('%.' + IntToStr(FDecimalDigits) +
                      'f', [FValue]                    );
   if FText <> OldText then begin
      SelLength := 0;
      Repaint;
      Change;
   end{if};
end;

procedure TD2SpinBox.SetText(const Value: WideString); // 7777
begin
   if FText <> Value then
   Try
      FValue := StrToFloat(Value);
   Except
   end{try};
   inherited;
end;

